/* ***********************************************************
hi_sync  Version 0.0.0
ローカルファイルバックアップ用ライブラリ

	_Dir : ファイルリスト所得クラス (need hi_files)
	C_bakSync : バックアップ実行クラス

2014-03-30
hi_tag	http://studyhall.webcrow.jp/
*********************************************************** */
library hi_sync;
import 'dart:core';
import 'dart:io';
import '../../hi_files/lib/hi_files.dart';

/**
 * Get file list and properties
 *
 *  Constructor
 *  	_Dir(this.basedir)  <lib:hi_files>
 *
 * 	Properties
 * 		option:	path of under the basedir
 *    flg: 0 = no use, 1 = checked, 2 = copied, 3 = updated, 4 = new, 5 = deleted
 */
class _Dir extends C_Files {
	String basedir;					// 比較のベースディレクトリ

	_Dir(this.basedir) {
		getList(basedir);

		// option にベースファイル以下のパスをセットする
		for (int i = 0; i < filename.length; i++) {
			String buff = directory[i];
			buff = buff.substring(basedir.length, buff.length);  // sBaseDir 以下を向き取り
			if (type[i] == 1) {
				buff += ('\\' + filename[i]);		// File の場合
			}
			/*
			print(i);
			print(directory[i]);
			print(filename[i]);
			print('type=' + type[i].toString());
			print('buff:' + buff);
			*/
			option[i] = buff;
			flg[i] = 0;
		}
	}

}

/**
 * Make local files backup
 *
 *  Constructor
 *  	C_bakSync(this.copypath, this.backpath)
 *  Metohds
 *  	int sync()
 *  		return: 0 = no error, -1 = Exception, -2 = other error
 *  Properties
 *  	String copypath: path of orignal files
 *  	String backpath: path of backup space
 *  	C_log log: log messages and save log file <lib:hi_files>
 */
class C_bakSync {
	String copypath;
	String backpath;
	C_log log = new C_log('baksync.log');

	C_bakSync(this.copypath, this.backpath);

	int sync() {
		int iErr = 0;
		String errMessage;
		_Dir copyfiles = new _Dir(copypath);
		_Dir backfiles = new _Dir(backpath);

		try {
			String buff;	// 一時処理用　log message にも使用
			// copyfiles を順次走査
			for(int i = 0; i < copyfiles.flg.length; i++) {
 				if (copyfiles.filename[i] == '.lock') {
 					// Microsoft OneDrive 対策
 					continue;
 				}
				int n = backfiles.option.indexOf(copyfiles.option[i]);
				if (n == -1) {
					// 存在しないとき、コピー
					switch (copyfiles.type[i]) {
						case 0:
							// Directory
							buff = backfiles.basedir + copyfiles.option[i];
							Directory targetDir = new Directory(buff);
							targetDir.createSync();
							log.add('Create:' + buff);
							break;
						case 1:
							// File
							File targetfile = new File(copyfiles.basedir + copyfiles.option[i]);
							buff = backfiles.basedir + copyfiles.option[i];
	    				targetfile.copySync(buff);
	    				log.add('Copy:' + buff);
	    				break;
	    			default:
	    				// backfilesに追加
	    				backfiles.option.add(copyfiles.option[i]);
	    				backfiles.flg.add(4);	// 新規作成
	    				copyfiles.flg[i] = 2; // copy 済み
					}
				} else {
					// Back先に同一ファイルまたはディレクトリが存在するとき
					backfiles.flg[n] = 1;	// 存在する
					if ((copyfiles.type[i] == 1)
							&& (backfiles.timeStamp[n].isBefore(copyfiles.timeStamp[i]))) {
						// Back先ファイルのほうが古いとき
						File targetfile = new File(copyfiles.basedir + copyfiles.option[i]);
						buff = backfiles.basedir + backfiles.option[n];
						targetfile.copySync(buff);
						log.add('Update:' + buff);
						copyfiles.flg[i] = 2;	// copy済
						backfiles.flg[n] = 3;	// update 済み
					} else {
						copyfiles.flg[i] = 1; // 処理済　(backup先には何もしない)
					}
				}
				// print('copyfile ' + copyfiles.option[i] + '/' +copyfiles.flg[i].toString());
			}

			// backfiles を走査
			for(int i = 0; i < backfiles.flg.length; i++) {
				if (backfiles.flg[i] == 0) {
					// 何も処理していない == copy元に存在しないから削除
					switch (backfiles.type[i]) {
						case 0:
							// Directory
							buff = backfiles.option[i];
							Directory targetDir = new Directory(buff);
							targetDir.deleteSync();
							log.add('Delete:' + buff);
							break;
						case 1:
							// File
							buff = backfiles.basedir + backfiles.option[i];
							File targetfile = new File(buff);
	    				targetfile.deleteSync(recursive: false);
	    				log.add('Delete:' + buff);
	    				break;
						default:
							backfiles.flg[i] = 5;	// 削除
					}
				}
			}
		} on Exception catch(e) {
      errMessage = e.toString();
      log.add(errMessage);
      iErr = -1;
		} catch(e) {
      errMessage = e.toString();
      log.add(errMessage);
      iErr = -2;
		}
		int iLogErr = log.write();
		if (iLogErr != 0) log.add('Log:' + log.errMessage);
		iErr = iLogErr * 10 + iErr;
		return iErr;
	}
}

